<?php

namespace NoccyLabs\TinyPhar;

class Updater
{
    protected $lists = [];

    protected $name;

    protected $version;

    protected $packages = [];

    public function readPackageList($uri)
    {
        if (in_array($uri, $this->lists)) {
            return;
        }
        $this->lists[] = $uri;
        $json = @file_get_contents($uri);
        if (!$json) {
            fprintf(STDERR, "tinyphar: Could not read channel from %s\n", $uri);
        } else {
            $this->parsePackages($json);
        }
    }

    protected function parsePackages($json)
    {
        $list = json_decode($json,true);
        foreach ($list as $package_name=>$package) {
            if (!array_key_exists($package_name,$this->packages)) {
                $this->packages[$package_name] = [];
            }
            $packages = [];
            foreach ($package['versions'] as $version=>$meta) {
                $packages[$version] = 
                    new Package($package_name, $version, $meta['url'], $meta['info']);
            }
            $packages = array_merge($this->packages[$package_name], $packages);
            uksort($packages, "version_compare");
            $this->packages[$package_name] = $packages;
        }
    }

    /**
     * Set the information about the package being updated directly from
     * strings.
     *
     * @param string $name
     * @param string $version
     */
    public function setPackageInfo($name, $version)
    {
        $this->name = $name;
        $this->version = $version;
    }

    /**
     * Get the latest version of the matching package.
     *
     * @param string $name Package name to match, defaults to $this->name
     * @return Package The latest version of the package
     */
    public function getLatestVersion($name=null)
    {
        $name = $name?:$this->name;
        if (empty($name)) {
            throw new UpdaterException("No package name to match against");
        }
        if (!array_key_exists($name, $this->packages)) {
            return false;
        }
        $versions = $this->packages[$name];
        return end($versions);
    }

    /**
     * Get all versions of the matching package.
     *
     * @param string $name Package name to match, defaults to $this->name
     * @return Package[] The available versions of the package
     */
    public function getAllVersions($name=null)
    {
        $name = $name?:$this->name;
        if (empty($name)) {
            throw new UpdaterException("No package name to match against");
        }
        if (!array_key_exists($name, $this->packages)) {
            return [];
        }
        return $this->packages[$name];
    }

    /**
     * Check if the current package is the latest version
     *
     * @return bool True if the current package is the latest version
     */
    public function isLatestVersion()
    {
        $latest = $this->getLatestVersion();
        if ($latest === false) {
            return true;
        }
        return (version_compare($this->version, $latest->getVersion()) >= 0);
    }

    /**
     * Get the names of all the packages in the loaded lists
     *
     * @return string[] The names of the available packages
     */
    public function getPackageNames()
    {
        return array_keys($this->packages);
    }

    public function downloadVersion(Package $package, $filename)
    {
        $url = $package->getUrl();
        
        $fdest = fopen($filename,"wb");
        $fsrc = fopen($url, "rb");
        @stream_copy_to_stream($fsrc, $fdest);
        @fclose($fsrc);
        @fclose($fdest);

        if (filesize($filename)>0) {
            return true;
        }
        return false;
    }

}

